function crm00050000_Interlocutor_Revenda_Controller(General, ScreenService, CRM_Utilities, crm00050000_Interlocutor_Revenda_Service, templateManager, ZHPromise) {
    
    /**
     * Valida a inscrição informada pelo usuário
     * e seta na currentRow da widget o código do
     * tipo da inscrição através do método 
     * 'setInscriptionTypeInCurrentRow', caso a 
     * inscrição seja inválida, ele limpa os campos
     * e emite uma mensagem.
     * 
     * @param {object} widget widget onde está sendo
     * realizado o cadastro do interlocutor.
    */
    this.validateInterlocutorInscription = function(widget){
        
        var currentRow = widget.currentRow;
        
        var resultValidate = General.inscricaoFormat(currentRow.NRINSCRICAOPARC);
        
        if(resultValidate.valid && resultValidate.value){
            
            this.setInscriptionTypeInCurrentRow(widget, resultValidate.tipo.cod);
            
            currentRow.NRINSCRICAOPARC = resultValidate.value;
            
        }else{
            
            this.showErrorMessageForInscription(currentRow.NRINSCRICAOPARC.length);
            
            currentRow.NRINSCRICAOPARC = "";
            currentRow.CDTIPOINSCRICAO = "";
        }
        
    };
    
    /**
     * Função para exibir a mensagem de erro referente
     * ao número de inscrição informado.
     * 
     * @param {int} inscription número de inscrição informado.
    */
    this.showErrorMessageForInscription = function(inscription){
    
        if(inscription){
    
            switch(inscription){
                
                case 11:
                    ScreenService.showMessage("Número de CPF inválido.");
                    break;
                case 14:
                    ScreenService.showMessage("Número de CNPJ inválido.");
                    break;
                default:
                    ScreenService.showMessage("Número de incrição inválido.");
            }
        
        }
        
    };
    
    /**
     * Seta na currentRow da widget, o código 
     * da inscrição informada.
     * 
     * @param {object} widget onde está sendo 
     * realizada o cadastro do interlocutor.
     * 
     * @param {int} inscriptionCod código da 
     * inscrição informada.
    */
    this.setInscriptionTypeInCurrentRow = function(widget, inscriptionCod){
        
        var currentRow = widget.currentRow;
        
        if (inscriptionCod){
            
            switch(inscriptionCod){
                
                case "F":
                    currentRow.CDTIPOINSCRICAO = "CPF";
                break;
                
                case "J":
                    currentRow.CDTIPOINSCRICAO = "CNPJ";
                break;
                
                case "O":
                    currentRow.CDTIPOINSCRICAO = "CODIGO";
                break;
                
                case "I":
                    currentRow.CDTIPOINSCRICAO = "LIVRE";
                break;
            }
            
        }
    };
    
    /**
     * Função para definir o procedimento para salver
     * as informações recebidas pelo formulário de registro
     * de interlocutor, podendo ser um atualização ou o
     * cadastro de um novo registro.
     * 
     * @param {object(widget)} widget widget ativa onde está
     * sendo realizado o cadastro ou a atualização de um
     * interlocutor.
    */
    this.saveResaleInterlocutor = function(widget){
        
        var currentRow = widget.currentRow;
        
        if(currentRow.__is_new){
            
            this.saveNewResaleInterlocutor(widget);
            
        }else{
            
            this.saveInterlocutorEdition(widget);
            
        }
            
    };
    
    /**
     * Função para salver um novo interlocutor
     * ligado a revenda em caso.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo realizado o cadastro do novo
     * interlocutor.
    */
    this.saveNewResaleInterlocutor = function(widget){
        
        var currentRow = widget.currentRow;
        var promise = ZHPromise.defer();
        
        this.setValuesToSaveInCurrentRow(widget);
        
        General.widgetSave(widget).then(function(response){
           
            var nrParcNegocio = response.dataset.columnsToReturn.NRPARCNEGOCIO;
           
            this.setPartnerNumberInCurrentRow(currentRow, nrParcNegocio);
            
            this.insertPartnerCommunication(widget);
            
            this.insertPartnerRelation(widget);
            
            this.insertPartnerParceria(widget);
            promise.resolve();
            
        }.bind(this));
        
        return promise.promise;
    };
    
    /**
     * Função para setar o número do parceiro 
     * cadastrado na currentRow da widget ativa.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo realizado o cadastro do interlocutor.
     * 
     * @param {int} nrParcNegocio número do parceiro cadastrado.
    */
    this.setPartnerNumberInCurrentRow = function(currentRow, nrParcNegocio){
        
        currentRow.NRPARCNEGOCIO = nrParcNegocio;
            
    };
    
    /**
     * Função para definir os valores necessários, na 
     * currentRow, para realizado o cadastro de um novo
     * interlocutor.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo realizado o cadastro de um novo
     * interlocutor.
    */
    this.setValuesToSaveInCurrentRow = function(widget){
        
        var currentRow = widget.currentRow;
        
        CRM_Utilities.setColumnToReturnInCurrentRow(currentRow, "NRPARCNEGOCIO");
        
        this.setInterlocutorSecundaryName(currentRow);
        
        this.setPersonType(currentRow);
        
        this.setPartnerPrincipalType(currentRow);
        
        this.setFusedPartnerId(currentRow);
        
        this.setPartnerDefaultState(currentRow);
            
    };
    
    /**
     * Função para igualar o nome secundário do 
     * interlocutor ao nome principal digitado pelo
     * usuário.
     * 
     * @param {object} currentRow currentRow do 
     * registro do novo interlocutor.
    */
    this.setInterlocutorSecundaryName = function(currentRow){
        
        var widgetPrincipal = templateManager.container.getWidget('WDGT_REVENDA');
        
        if(currentRow.NMPRINCIPALPARC){
        
            currentRow.NMSECUNDARIPARC = 'Referente a Revenda '+widgetPrincipal.currentRow.NMRAZSOCREVENDA;
            
        }
        
    };
    
    /**
     * Função para definir se a inscrição informada
     * é uma inscrição de pessoa física ou jurídica.
     * 
     * @param {object} currentRow currentRow do 
     * registro do novo interlocutor.
    */
    this.setPersonType = function(currentRow){

        var inscription = currentRow.NRINSCRICAOPARC;
        var inscriptionWithoutSymbols = inscription.replace(/[^0-9]/g, "");
        
        if(inscriptionWithoutSymbols.length === 11){
            
            currentRow.IDPESSOAFISICA = "S"; 
            currentRow.IDINSTITUICAO = "N";
            
        }else if(inscriptionWithoutSymbols .length === 14){
            
            currentRow.IDPESSOAFISICA = "N"; 
            currentRow.IDINSTITUICAO = "S";
            
        }else{
            
            currentRow.IDPESSOAFISICA = "N"; 
            currentRow.IDINSTITUICAO = "N";
            
        }
        
        
    };
    
    /**
     * Função para setar o tipo do parceiro
     * que será cadastrado.
     * 
     * @param {object} currentRow currentRow do 
     * registro do novo interlocutor.
    */
    this.setPartnerPrincipalType = function(currentRow){
        
        currentRow.CDTIPOPARCPRINCIPAL = "INTERLOCUTOR";
        
    };
    
    /**
     * Função para definir que o parceiro não
     * está sendo mesclado com nenhum outro
     * parceiro.
     * 
     * @param {object} currentRow currentRow do 
     * registro do novo interlocutor.
    */
    this.setFusedPartnerId = function(currentRow){
        
        currentRow.IDPARCFUNDIDO = "N";
        
    };
    
    /**
     * Função para setar o valor inicial do estado
     * do novo parceiro.
     * 
     * @param {object} currentRow currentRow do 
     * registro do novo interlocutor.
    */
    this.setPartnerDefaultState = function(currentRow){
        currentRow.IDATIVO = "S";
    };
    
    /**
     * Função para chamar funções que serão executadas
     * no evento 'New Row' da widget de interlocutor.
     * 
     * @param {object(widget)} widget widget de interlocutor.
    */
    this.interlocutorWidgetNewRow = function(widget){
      
        this.setLoggedOperatorInfosInCurrentRow(widget);
        
    };
    
    
    /**
     * Função para setar as informações do operador
     * logado na currentRow da widget recebida como 
     * parâmetro.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo realizado o cadastro de um novo
     * interlocutor.
    */
    this.setLoggedOperatorInfosInCurrentRow = function(widget){
      
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        var currentRow = widget.currentRow;
      
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        currentRow.CDOPERADOR = loggedOperatorInfos.CDOPERADOR;
      
        
    };
    
    /**
     * Função para inserir os meios de comunicação
     * do interlocutor cadastrado.
     * 
     * @param {object(widget)} widget widget ativa onde
     * está sendo feito o cadastro do interlocutor.
    */
    this.insertPartnerCommunication = function(widget){
        
        var currentRow;
        
        if(widget.currentRow){
            currentRow = widget.currentRow;
        }else{
            currentRow = widget;
        }
        
        if(currentRow.NRTELEFONEINTERLOC){
            
            this.insertPartnerPhone(widget).then(function(){
                
                if(currentRow.DSEMAILINTERLOC){
            
                    this.insertPartnerEmail(widget);
                    
                }
                    
            }.bind(this));
            
        }else if(currentRow.DSEMAILINTERLOC){
            
            this.insertPartnerEmail(widget);
            
        }
        
    };
    
    /**
     * Função para insetir o número de telefone
     * do interlocutor em cadastro.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo feito o cadastro do interlocutor.
    */
    this.insertPartnerPhone = function(widget){
        
        var currentRow;
        
        if(widget.currentRow){
            currentRow = widget.currentRow;
        }else{
            currentRow = widget;
        }
        var interlocutorPhone = this.removePhoneMask(currentRow.NRTELEFONEINTERLOC);
        
        this.setPartnerCommunicationInCurrentRow(currentRow, interlocutorPhone);
        this.setPartnerCommunicationTypeInCurrentRow(currentRow, "01");
        
        return crm00050000_Interlocutor_Revenda_Service.callForInsertPartnerCommunication(widget);
        
    };
    
    /**
     * Função para remover a máscara do 
     * número de telefone recebido como
     * parâmetro.
     * 
     * @param {string} phone número de telefone
     * com a máscara.
    */
    this.removePhoneMask = function(phone){
        
        return phone.replace(/[^0-9]/g, "");
        
    };
    
    /**
     * Função para insetir o E-Mail 
     * do interlocutor em cadastro.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo feito o cadastro do interlocutor.
    */
    this.insertPartnerEmail = function(widget){
        
        var currentRow;
        
        if(widget.currentRow){
            currentRow = widget.currentRow;
        }else{
            currentRow = widget;
        }
        
        this.setPartnerCommunicationInCurrentRow(currentRow, currentRow.DSEMAILINTERLOC);
        this.setPartnerCommunicationTypeInCurrentRow(currentRow, "05");
        
        crm00050000_Interlocutor_Revenda_Service.callForInsertPartnerCommunication(widget);
        
    };
    
    /**
     * Função para definir a comunicação do
     * interlocutor na currentRow do cadastro.
     * 
     * @param {object} currentRow currentRow do cadastro
     * do interlocutor.
     * 
     * @param {string} communication comunicação do 
     * interlocutor.
    */
    this.setPartnerCommunicationInCurrentRow = function(currentRow, communication){
        
        currentRow.CDCOMUNICAPARC = communication;
        
    };
    
    /**
     * Função para definir a forma de comunicação do
     * interlocutor na currentRow do cadastro.
     * 
     * @param {object} currentRow currentRow do cadastro
     * do interlocutor.
     * 
     * @param {string} codigo da formaa da communication 
     * comunicação do interlocutor.
    */
    this.setPartnerCommunicationTypeInCurrentRow = function(currentRow, cdCommunication){
        
        currentRow.CDFORMACOMU = cdCommunication;
        
    };
    
    /**
     * Função para inserir um relacionamento
     * de parceiros.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo feito o cadastro do interlocutor.
    */
    this.insertPartnerRelation = function(widget){
        
        this.getResalePartnerNumber(widget);
        
        crm00050000_Interlocutor_Revenda_Service.callForInsertPartnerRelation(widget);
        
    };
    
    /**
     * Função para inserir um relacionamento
     * de parceiros.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo feito o cadastro do interlocutor.
    */
    this.insertPartnerParceria = function(widget){
        
        this.getResalePartnerNumber(widget);
        
        crm00050000_Interlocutor_Revenda_Service.callForInsertPartnerParceria(widget);
        
    };
    
    /**
     * Função para setar na currentRow de cadastro
     * do novo interlocutor o número da revenda em
     * cado.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo feito o cadastro do interlocutor.
    */
    this.getResalePartnerNumber = function(widget){
    
        var currentRow = widget.currentRow;
        
        currentRow.NRPARCNEGOCIOREV = widget.parent.parent.currentRow.NRPARCNEGOCIO;
        
    };
    
    /**
     * Função para setar o filtro da widget
     * de 'INTERLOCUTOR';
     * 
     * @param {object(widget)} widget widget ativa
     * ligada a revenda em caso.
    */
    this.setInterlocutorWidgetFilter = function(widget){
        
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        var interlocutorWidget = templateManager.container.getWidget('WDGT_INTERLUCUTOR');
        var interlocutorWidgetFilter = interlocutorWidget.dataSourceFilter;
        
        var nrorgFilterIndex = CRM_Utilities.getFilterPosition(interlocutorWidgetFilter, "P_NRORG");
        var nrParcRevendaFilterIndex = CRM_Utilities.getFilterPosition(interlocutorWidgetFilter, "P_PARCREVENDA");
        
        interlocutorWidgetFilter[nrorgFilterIndex].value = loggedOperatorInfos.NRORG;
        interlocutorWidgetFilter[nrParcRevendaFilterIndex].value = widget.currentRow.NRPARCNEGOCIO;
            
    };
    
    /**
     * Função para salvar as edições feitas
     * no interlocutor selecionado.
     * 
     * @param {object(widget)} widget widget ativa 
     * onde está sendo feita a edição do interlocutor.
    */
    this.saveInterlocutorEdition = function(widget){
        
        this.updateRelationState(widget);
        
        this.deletePartnerCommunication(widget).then(function(){
            
            this.insertPartnerCommunication(widget);
            
            General.widgetSave(widget);
                
        }.bind(this));
            
    };
    
    /**
     * Função para deletar uma comunicação, no ambiente
     * de edição, ligada ao interlocutor em casoç.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo realizado a edição de um 
     * interlocutor.
    */
    this.deletePartnerCommunication = function(widget){
        
        return crm00050000_Interlocutor_Revenda_Service.callForDeletePartnerCommunication(widget);

    };
    
    /**
     * Função para salvar o estado do relacionamento
     * entre o interlocutor em edição e a revenda 
     * em caso.
     * 
     * @param {object(widget)} widget widget ativa
     * onde está sendo realizado a edição de um 
     * interlocutor.
    */
    this.updateRelationState = function(widget){
        
        this.getResalePartnerNumber(widget);
        
        crm00050000_Interlocutor_Revenda_Service.callForUpdateRelationState(widget);
        
    };
    
    /**
     * Função para deletar um interlocutor.
     * 
     * @param {object(widget)} widget widget ativa
     * , no modo 'view', mostrando o interlocutor
     * selecionado.
    */
    this.deleteResaleInterlocutor = function(widget){
        
        this.inactivatePartner(widget);
        
        this.deleteResaleRelation(widget).then(function(){
            
            ScreenService.closeSwipe().then(function(){
               
                widget.list();
                widget.reload();
                
            });
            
        });
        
    };
    
    this.inactivatePartner = function(widget){
        
        crm00050000_Interlocutor_Revenda_Service.callForInactivatePartner(widget);
        
    };
    
    this.deleteResaleRelation = function(widget){
        
        this.getResalePartnerNumber(widget);
        
        return crm00050000_Interlocutor_Revenda_Service.callForDeleteResaleRelation(widget);
        
    };
    
}

Configuration(function (ContextRegister) {
    ContextRegister.register("crm00050000_Interlocutor_Revenda_Controller", crm00050000_Interlocutor_Revenda_Controller);
});
