/* global UtilDevTools, Configuration, openWindow */
function SocketService(ZhSocketFactory, ZHPromise, ScreenService, templateManager, restEngine, ApplicationContext, DevToolsService, DTMessages, DTRenderService){
    var self = this;
    var CLIENT = 'browser';
    var NAMESPACE = 'zh_developer';
    var WSKEY = 'you can create!';
    var defaultHost = 'ws://ws.zeedhi.com';
    var isConnected = false;
    var webSocket = null;
    var clientEmail = null;
    var inDeveloperMode = false;
    var ERROR_TYPE = 'error';
    var SUCCESS_TYPE = 'success';

    var pendingChanges = {};

    var getWindowController = function(){
        return ApplicationContext.WindowController;
    };

    var getLoginData = function(email){
        var login = {
            data: {}
        };
        login.id = (new Date().getTime());
        login.sender = CLIENT;
        if(email){
            var defer = ZHPromise.defer();
            clientEmail = login.login = email;
            defer.resolve(login);
            return defer.promise;
        } else {
            return restEngine.requestMetaData('.developerConfig.json').then(function(obj){
                clientEmail = login.login = obj.data.email;
                DevToolsService.setLocalEmail(login.login);
                return login;
            });
        }
    };

    var formatErrorMessage = function(path, code){
        var message = "Error while writing " + path;
        if(DTMessages.FILE_ERROR_MAP[code]){
            message += " : " + DTMessages.FILE_ERROR_MAP[code];
        }
        return message;
    };

    var getContainerComming = function(data){
        var containerComming;
        try {
            containerComming = JSON.parse(data);
        } catch(e){
            var message = ~data.indexOf("Syntax error") ? data : e;
            ScreenService.notificationMessage(message, 'error', 5000);
        }
        return containerComming;
    };

    var isJsFile = function(fileName){
        return fileName.indexOf(".json") == -1 && fileName.indexOf(".js") != -1;
    };

    var sendMessageToServer = function(messageToSend, ignoreResponse){
        var data = messageToSend.data || {};
        var dataId = messageToSend.id;

        messageToSend.room = clientEmail;
        messageToSend.namespace = NAMESPACE;

        data.id = dataId;
        data.namespace = NAMESPACE;
        data.key = WSKEY;
        webSocket.send(JSON.stringify(messageToSend));

        if(ignoreResponse){
            return ZHPromise.when();
        } else {
            var defer = ZHPromise.defer();
            pendingChanges[dataId] = {
                defer: defer,
                id: dataId
            };
            setTimeout(function(){
                if(pendingChanges[dataId]){
                    defer.reject({
                        "path": "",
                        "code": "ETIMEDOUT"
                    });
                }
            }, 15000);
            return defer.promise;
        }
    };

    var getPreparedPathname = function(){
        var pathname = window.location.pathname;
        var lastSlash = pathname.lastIndexOf('/');
        pathname = ~lastSlash ? pathname.slice(0, lastSlash) : pathname;
        return pathname;
    };

    var getMessageIndexPath = function(indexPath, pathname){
        if(pathname[0] == '/'){
            pathname = pathname.replace('/', '');
        }
        indexPath = indexPath.replace(pathname, "");
        return indexPath;
    };

    var getFileName = function(data){
        return data.fileName || data.__indexPath.split("/").pop();
    };

    var getDataToSend = function(data, mode){
        var eventName = 'changesByBrowser';
        var pathname = getPreparedPathname();
        var messageData = {
            mode: mode || 'json',
            fileName: getFileName(data),
            projectPath: pathname,
            indexPath: data.__indexPath,
            sender: CLIENT,
            email: DevToolsService.getLocalEmail(),
            project: DevToolsService.getProject(),
            event: eventName
        };
        delete data.fileName;
        delete data.__indexPath;
        messageData.data = mode == 'files' ? data.data : data;
        messageData.projectPath = pathname;
        messageData.indexPath = getMessageIndexPath(messageData.indexPath, pathname);
        var dataToSend = {
            id: (new Date().getTime()),
            data: messageData,
            event: eventName
        };
        return dataToSend;
    };

    var showNotificationInDevTools = function(messageToShow, type){
        var windowController = getWindowController();
        if(windowController && windowController.isDevToolsOpened()){
            windowController.showNotificationInDevTools(messageToShow, type);
        }
    };

    var showMessageInWindows = function(messageToMain, messageToDevTools, type, showDialog){
        if(type != ERROR_TYPE && !showDialog){
            ScreenService[type+"Notification"](messageToMain, type);
        } else {
            ScreenService.showMessage(messageToMain, type);
        }
        showNotificationInDevTools(messageToDevTools, type);
    };

    var getBasicDataToSend = function(data, eventName){
        data = Util.isObject(data) ? data : {data: data};
        data.sender = CLIENT;
        data.event = eventName;
        data.project = DevToolsService.getProject();
        data.email = DevToolsService.getLocalEmail();

        var dataToSend = {
            id: (new Date().getTime()),
            event: eventName,
            data: data,
        };
        return dataToSend;
    };

    var sendDataLogin = function(login){
        setTimeout(function(){
            var eventsToStart = ['createNamespace', 'createRoom', 'joinRoom'];
            eventsToStart.forEach(function(event){
                login.data.event = login.event = event;
                sendMessageToServer(login, true);
            });
            login.data.event = login.event = "loginClient";

            sendMessageToServer(login).then(function(){
                ScreenService.successNotification(DTMessages.LOGIN_ACCOMPLISHED);
                sendPathToWatch(login);
                self.updateScriptTreeWhenReloaded();
            }).catch(function(err){
                ScreenService.showMessage(DTMessages.LOGIN_NOT_ACCOMPLISHED, 'error');
            });
        }, 1000);
    };
    var sendPathToWatch = function(dataToSend){
        var pathname= window.location.href.replace(window.location.origin,"");
            pathname = pathname.slice(0, pathname.indexOf(window.location.hash));
            pathname = pathname.slice(0, pathname.lastIndexOf('/'));
        dataToSend.data.projectPath = pathname;
        dataToSend.data.event = dataToSend.event = 'sendPathToWatch';
        sendMessageToServer(dataToSend, true);
    };
    var getHost = function(host){
        if(!host.match('ws://') || !host){
            host = defaultHost;
        }
        return host;
    };
    var getEmailByClient = function(client){
        if(client && client.email){
            client = client.email;
        }
        return client;
    };

    var findFiles = function(data, callback){
        data = data || 'new';
        var eventName = 'findFiles';
        var dataToSend = getBasicDataToSend(data, eventName);
        setTimeout(function(){
            sendMessageToServer(dataToSend).then(function(responseData){
                if(callback){
                    callback(responseData);
                }
            }).catch(function(){
                ScreenService.showMessage(DTMessages.LOGIN_NOT_ACCOMPLISHED, 'error');
            });
        }, 1000);
    };

    var ResponseHandler = function(){
        this.receiveChanges = UtilDevTools.buildMainWindowMethod(function(data){
            var containerComming = getContainerComming(data);
            if(containerComming){
                if(templateManager.container && templateManager.container.name === containerComming.name){
                    DTRenderService.renderContainer(containerComming);
                } else if(containerComming.zeedhi_project){
                    var project = containerComming.zeedhi_project;
                    DTRenderService.updateProjectConfig(project);
                } else if(containerComming.fileName && isJsFile(containerComming.fileName)){
                    getWindowController().reloadMainWindow();
                }
            }
        });

        this.loginClient = function(data){
            data.success = true;
            this.response(data);
        };

        this.noOtherClientsConnected = function(){
            Object.keys(pendingChanges).forEach(function(pendingChangeId){
                pendingChanges[pendingChangeId].reject({});
                delete pendingChanges[pendingChangeId];
            });
            ScreenService.alertNotification(DTMessages.LOGIN_NO_OTHER_CLIENTS);
        };

        this.response = function(data){
            var responseId = data.id;
            var responseData = data.data || {};
            var method = data.isError ? 'reject' : 'resolve';
            if(pendingChanges[responseId]){
                responseData.id = responseId;
                pendingChanges[responseId].defer[method](responseData);
                delete pendingChanges[responseId];
            }
        };
    };

    this.updateScriptTreeWhenReloaded = function(){
        getWindowController().updateScriptTree();
    };

    this.reloadFiles = function(){
        findFiles('reload', getWindowController().setScriptsFiles);
    };

    this.loadFiles = function(){
        findFiles('new', getWindowController().setFilesToLocalStorage);
    };

    this.newFile = UtilDevTools.buildMainWindowMethod(function(message){
        if(message){
            var fileName = message.fileName.replace(".json", "");
            var dataToSend = {
                id: (new Date().getTime()),
                event: "newFile",
                data: {
                    data: message,
                    sender: CLIENT,
                    email: DevToolsService.getLocalEmail(),
                    project: DevToolsService.getProject(),
                    event: "newFile"
                }
            };
            sendMessageToServer(dataToSend).then(function(){
                var messageToShow = DTMessages.CONTAINER_WAS_CREATED;
                showMessageInWindows(messageToShow, messageToShow, SUCCESS_TYPE);
                openWindow(fileName);
            }).catch(function(err){
                showMessageInWindows(formatErrorMessage(err.path, err.code), DTMessages.CHECK_MAIN_WINDOW, ERROR_TYPE);
            });
        }
    });

    this.changeInClient = UtilDevTools.buildMainWindowMethod(function(data, mode){
        var isInOtherPackage = data.__indexPath.indexOf('http://') != -1 || data.__indexPath.indexOf('https://') != -1;
        if(isInOtherPackage){
            showMessageInWindows(DTMessages.FILE_CANT_SAVE, DTMessages.FILE_CANT_SAVE, ERROR_TYPE);
        } else {
            sendMessageToServer(getDataToSend(data, mode)).then(function(){
                showMessageInWindows(DTMessages.FILE_SAVED_SUCCESSFULLY, DTMessages.FILE_SAVED_SUCCESSFULLY, SUCCESS_TYPE);
            }).catch(function(err){
                showMessageInWindows(formatErrorMessage(err.path, err.code), DTMessages.CHECK_MAIN_WINDOW, ERROR_TYPE);
            });
        }
    });

    this.login = UtilDevTools.buildMainWindowMethod(function(email){
        getLoginData(email).then(function(login){
            if(isConnected){
                sendDataLogin(login);
            } else {
                webSocket.retryConnection();
                sendDataLogin(login);
            }
        }).catch(function(err){
            ScreenService.showMessage(err.status + " : File .developerConfig.json " + err.statusText, 'error');
        });
        DevToolsService.setProject();
    });

    this.connect = UtilDevTools.buildMainWindowMethod(function(client){
        DevToolsService.inDeveloperModePromise.then(function(definedInDeveloperMode){
            inDeveloperMode = definedInDeveloperMode;
            if(inDeveloperMode || client){
                var email = getEmailByClient(client);
                webSocket = ZhSocketFactory.create(ResponseHandler, getHost(templateManager.project.developerWebSocketUrl));
                webSocket.addListener({
                    onOpen: function(){
                        isConnected = true;
                        self.login(email);
                    }
                },{
                    connectionLost: function(){
                        isConnected = false;
                    }
                });
            }
        });
    });
    this.connect();

    window.SocketServiceInstance = this;
}
